local http = require "resty.http"

local config = require "config"
local geo = require "geo"
local utils = require "utils"

local jasima_host = config.get_jasima_host()
ngx.ctx.jasima_host = jasima_host
if not jasima_host then
    return ngx.exit(400)
end

local err
ngx.ctx.jasima_config, err = config.load_config(jasima_host)
if err then
    ngx.status = 500
    ngx.log(ngx.ERR, "Could not load config: " .. err)
    return ngx.exit(500)
end

if not ngx.ctx.jasima_config then
    ngx.status = 403
    ngx.log(ngx.ERR, "Requested a canonical host that has no configuration specified: " .. jasima_host)
    ngx.exit(403)
end

local country = geo.viewer_country()
if not ngx.ctx.jasima_config.geo_redirect_disable and not geo.needs_mirror(country) then
    local request_uri = ngx.var.request_uri
    local new_url = "https://" .. jasima_host .. request_uri
    return ngx.redirect(new_url, ngx.HTTP_MOVED_TEMPORARILY)
end

-- Get jasima_pool (not critical)
local jasima_pool = config.get_jasima_pool()
ngx.ctx.jasima_pool_map, err = config.load_pool_mapping(jasima_pool)
if err then
    ngx.status = 500
    ngx.log(ngx.WARN, "Could not load pool mapping: " .. err)
    return ngx.exit(500)
end

local headers = ngx.req.get_headers()

-- Remove the headers that should not be proxied to the origin
for k, v in pairs(headers) do
  if k:lower():match("^jasima%-") then
    ngx.req.clear_header(k)
  end
end

-- Add additional headers that have been specified in the configuration
if ngx.ctx.jasima_config.headers then
    for k, v in pairs(ngx.ctx.jasima_config.headers) do
        ngx.req.set_header(k, v)
    end
end

-- Look up the IP to connect to the origin
local host_connect = ngx.ctx.jasima_config.host_connect or jasima_host
local upstream_ips = utils.resolve_origin(host_connect)
ngx.ctx.upstream_ips = utils.filter_bogons(upstream_ips)

if #ngx.ctx.upstream_ips == 0 then
    ngx.log(ngx.ERR, "no A records found")
    return ngx.exit(500)
end

-- Set the nginx host variables
ngx.var.jasima_host = jasima_host
ngx.var.jasima_host_header = ngx.ctx.jasima_config.host_header or jasima_host
ngx.var.jasima_host_ssl = ngx.ctx.jasima_config.host_ssl or jasima_host
